@@ -26,56 +26,10 @@
 #include "gva-tree-view.h"
 #include "gva-ui.h"
 
+#define MAX_PLAYER_ICONS 8
+
 typedef GtkTreeViewColumn * (*FactoryFunc) (GvaGameStoreColumn);
 
-static void
-columns_favorite_clicked_cb (GvaCellRendererPixbuf *renderer,
-                             GtkTreePath *path,
-                             GtkTreeViewColumn *column)
-{
-        GtkWidget *widget;
-        const gchar *name;
-
-        /* The row that was clicked is not yet selected.  We need to
-         * select it first so that gva_tree_view_get_selected_game()
-         * returns the correct name. */
-        widget = column->tree_view;
-        /*widget = gtk_tree_view_column_get_tree_view (column);*/
-        gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
-        gtk_widget_grab_focus (widget);
-
-        name = gva_tree_view_get_selected_game ();
-        g_assert (name != NULL);
-
-        if (gva_favorites_contains (name))
-                gtk_action_activate (GVA_ACTION_REMOVE_FAVORITE);
-        else
-                gtk_action_activate (GVA_ACTION_INSERT_FAVORITE);
-}
-
-static void
-columns_sampleset_set_properties (GtkTreeViewColumn *column,
-                                  GtkCellRenderer *renderer,
-                                  GtkTreeModel *model,
-                                  GtkTreeIter *iter)
-{
-        GvaGameStoreColumn column_id;
-        gboolean sensitive;
-        gboolean visible;
-        gchar *sampleset;
-
-        column_id = gtk_tree_view_column_get_sort_column_id (column);
-        gtk_tree_model_get (model, iter, column_id, &sampleset, -1);
-
-        visible = (sampleset != NULL && *sampleset != '\0');
-        sensitive = visible && (strcmp (sampleset, "good") == 0);
-
-        g_object_set (
-                renderer, "sensitive", sensitive, "visible", visible, NULL);
-
-        g_free (sampleset);
-}
-
 static GdkPixbuf *
 columns_get_icon_name (const gchar *icon_name)
 {
@@ -107,20 +61,126 @@ columns_get_icon_name (const gchar *icon_name)
         return scaled;
 }
 
+static void
+columns_favorite_clicked_cb (GvaCellRendererPixbuf *renderer,
+                             GtkTreePath *path,
+                             GtkTreeViewColumn *column)
+{
+        GtkWidget *widget;
+        const gchar *name;
+
+        /* The row that was clicked is not yet selected.  We need to
+         * select it first so that gva_tree_view_get_selected_game()
+         * returns the correct name. */
+#if GTK_CHECK_VERSION (2, 12, 0)
+        widget = gtk_tree_view_column_get_tree_view (column);
+#else
+        widget = column->tree_view;
+#endif
+        gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
+        gtk_widget_grab_focus (widget);
+
+        name = gva_tree_view_get_selected_game ();
+        g_assert (name != NULL);
+
+        if (gva_favorites_contains (name))
+                gtk_action_activate (GVA_ACTION_REMOVE_FAVORITE);
+        else
+                gtk_action_activate (GVA_ACTION_INSERT_FAVORITE);
+}
+
+static void
+columns_driver_status_set_properties (GtkTreeViewColumn *column,
+                                      GtkCellRenderer *renderer,
+                                      GtkTreeModel *model,
+                                      GtkTreeIter *iter)
+{
+        GvaGameStoreColumn column_id;
+        GdkPixbuf *pixbuf = NULL;
+        gchar *status;
+
+        column_id = gtk_tree_view_column_get_sort_column_id (column);
+        gtk_tree_model_get (model, iter, column_id, &status, -1);
+
+        /* XXX For the "good" icon I want some kind of positive symbol; a
+         *     checkmark or thumbs up or something.  The "emblem-default"
+         *     image in gnome-icon-theme seems to be the closest match at
+         *     this time (a green checkmark), but may not be suitable for
+         *     all icon themes. */
+
+        if (strcmp (status, "good") == 0)
+                pixbuf = columns_get_icon_name ("emblem-default");
+        else if (strcmp (status, "imperfect") == 0)
+                pixbuf = columns_get_icon_name ("dialog-warning");
+        else if (strcmp (status, "preliminary") == 0)
+                pixbuf = columns_get_icon_name ("dialog-error");
+
+        g_object_set (
+                renderer, "pixbuf", pixbuf,
+                "visible", (pixbuf != NULL), NULL);
+
+        if (pixbuf != NULL)
+                g_object_unref (pixbuf);
+
+        g_free (status);
+}
+
+static void
+columns_input_players_set_properties (GtkTreeViewColumn *column,
+                                      GtkCellRenderer *renderer,
+                                      GtkTreeModel *model,
+                                      GtkTreeIter *iter,
+                                      gpointer user_data)
+{
+        GvaGameStoreColumn column_id;
+        gint max_players;
+        gboolean visible;
+
+        column_id = gtk_tree_view_column_get_sort_column_id (column);
+        gtk_tree_model_get (model, iter, column_id, &max_players, -1);
+
+        visible = GPOINTER_TO_INT (user_data) < max_players;
+        g_object_set (renderer, "visible", visible, NULL);
+}
+
+static void
+columns_sampleset_set_properties (GtkTreeViewColumn *column,
+                                  GtkCellRenderer *renderer,
+                                  GtkTreeModel *model,
+                                  GtkTreeIter *iter)
+{
+        GvaGameStoreColumn column_id;
+        gboolean sensitive;
+        gboolean visible;
+        gchar *sampleset;
+
+        column_id = gtk_tree_view_column_get_sort_column_id (column);
+        gtk_tree_model_get (model, iter, column_id, &sampleset, -1);
+
+        visible = (sampleset != NULL && *sampleset != '\0');
+        sensitive = visible && (strcmp (sampleset, "good") == 0);
+
+        g_object_set (
+                renderer, "sensitive", sensitive, "visible", visible, NULL);
+
+        g_free (sampleset);
+}
+
 static GtkTreeViewColumn *
 columns_factory_description (GvaGameStoreColumn column_id)
 {
-        GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
-
-        renderer = gtk_cell_renderer_text_new ();
-        g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+        GtkCellRenderer *renderer;
 
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_expand (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, column_id);
+        gtk_tree_view_column_set_spacing (column, 3);
         gtk_tree_view_column_set_title (column, _("Title"));
+
+        renderer = gtk_cell_renderer_text_new ();
+        g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
         gtk_tree_view_column_add_attribute (
@@ -129,22 +189,43 @@ columns_factory_description (GvaGameStoreColumn column_id)
         return column;
 }
 
+static GtkTreeViewColumn *
+columns_factory_driver_status (GvaGameStoreColumn column_id)
+{
+        GtkTreeViewColumn *column;
+        GtkCellRenderer *renderer;
+
+        column = gtk_tree_view_column_new ();
+        gtk_tree_view_column_set_reorderable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, column_id);
+        gtk_tree_view_column_set_title (column, _("Status"));
+
+        renderer = gtk_cell_renderer_pixbuf_new ();
+        gtk_tree_view_column_pack_start (column, renderer, TRUE);
+
+        gtk_tree_view_column_set_cell_data_func (
+                column, renderer, (GtkTreeCellDataFunc)
+                columns_driver_status_set_properties, NULL, NULL);
+
+        return column;
+}
+
 static GtkTreeViewColumn *
 columns_factory_favorite (GvaGameStoreColumn column_id)
 {
-        GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
+        GtkCellRenderer *renderer;
         GdkPixbuf *pixbuf;
 
         pixbuf = columns_get_icon_name ("emblem-favorite");
 
-        renderer = gva_cell_renderer_pixbuf_new ();
-        g_object_set (renderer, "pixbuf", pixbuf, NULL);
-
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, column_id);
         gtk_tree_view_column_set_title (column, _("Favorite"));
+
+        renderer = gva_cell_renderer_pixbuf_new ();
+        g_object_set (renderer, "pixbuf", pixbuf, NULL);
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
         gtk_tree_view_column_add_attribute (
@@ -160,18 +241,52 @@ columns_factory_favorite (GvaGameStoreColumn column_id)
         return column;
 }
 
+static GtkTreeViewColumn *
+columns_factory_input_players (GvaGameStoreColumn column_id)
+{
+        GtkTreeViewColumn *column;
+        GdkPixbuf *pixbuf;
+        gint ii;
+
+        pixbuf = columns_get_icon_name ("stock_person");
+
+        column = gtk_tree_view_column_new ();
+        gtk_tree_view_column_set_reorderable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, column_id);
+        gtk_tree_view_column_set_title (column, _("Players"));
+
+        for (ii = 0; ii < MAX_PLAYER_ICONS; ii++)
+        {
+                GtkCellRenderer *renderer;
+
+                renderer = gtk_cell_renderer_pixbuf_new ();
+                g_object_set (renderer, "pixbuf", pixbuf, NULL);
+                gtk_tree_view_column_pack_start (column, renderer, FALSE);
+
+                gtk_tree_view_column_set_cell_data_func (
+                        column, renderer, (GtkTreeCellDataFunc)
+                        columns_input_players_set_properties,
+                        GINT_TO_POINTER (ii), NULL);
+        }
+
+        if (pixbuf != NULL)
+                g_object_unref (pixbuf);
+
+        return column;
+}
+
 static GtkTreeViewColumn *
 columns_factory_manufacturer (GvaGameStoreColumn column_id)
 {
-        GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
-
-        renderer = gtk_cell_renderer_text_new ();
+        GtkCellRenderer *renderer;
 
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, column_id);
         gtk_tree_view_column_set_title (column, _("Manufacturer"));
+
+        renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
         gtk_tree_view_column_add_attribute (
@@ -183,15 +298,15 @@ columns_factory_manufacturer (GvaGameStoreColumn column_id)
 static GtkTreeViewColumn *
 columns_factory_name (GvaGameStoreColumn column_id)
 {
-        GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
-
-        renderer = gtk_cell_renderer_text_new ();
+        GtkCellRenderer *renderer;
 
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, column_id);
         gtk_tree_view_column_set_title (column, _("ROM Name"));
+
+        renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
         gtk_tree_view_column_add_attribute (
@@ -203,19 +318,19 @@ columns_factory_name (GvaGameStoreColumn column_id)
 static GtkTreeViewColumn *
 columns_factory_sampleset (GvaGameStoreColumn column_id)
 {
-        GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
+        GtkCellRenderer *renderer;
         GdkPixbuf *pixbuf;
 
         pixbuf = columns_get_icon_name ("emblem-sound");
 
-        renderer = gtk_cell_renderer_pixbuf_new ();
-        g_object_set (renderer, "pixbuf", pixbuf, NULL);
-
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, column_id);
         gtk_tree_view_column_set_title (column, _("Samples"));
+
+        renderer = gtk_cell_renderer_pixbuf_new ();
+        g_object_set (renderer, "pixbuf", pixbuf, NULL);
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
         gtk_tree_view_column_set_cell_data_func (
@@ -229,17 +344,37 @@ columns_factory_sampleset (GvaGameStoreColumn column_id)
 }
 
 static GtkTreeViewColumn *
-columns_factory_year (GvaGameStoreColumn column_id)
+columns_factory_sourcefile (GvaGameStoreColumn column_id)
 {
-        GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
+        GtkCellRenderer *renderer;
+
+        column = gtk_tree_view_column_new ();
+        gtk_tree_view_column_set_reorderable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, column_id);
+        gtk_tree_view_column_set_title (column, _("Driver"));
 
         renderer = gtk_cell_renderer_text_new ();
+        gtk_tree_view_column_pack_start (column, renderer, TRUE);
+
+        gtk_tree_view_column_add_attribute (
+                column, renderer, "text", column_id);
+
+        return column;
+}
+
+static GtkTreeViewColumn *
+columns_factory_year (GvaGameStoreColumn column_id)
+{
+        GtkTreeViewColumn *column;
+        GtkCellRenderer *renderer;
 
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, column_id);
         gtk_tree_view_column_set_title (column, _("Year"));
+
+        renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
         gtk_tree_view_column_add_attribute (
@@ -257,7 +392,7 @@ column_info[GVA_GAME_STORE_NUM_COLUMNS] =
 {
         { "name",               columns_factory_name },
         { "favorite",           columns_factory_favorite },
-        { "sourcefile",         NULL },
+        { "sourcefile",         columns_factory_sourcefile },
         { "runnable",           NULL },
         { "cloneof",            NULL },
         { "romof",              NULL },
@@ -270,10 +405,10 @@ column_info[GVA_GAME_STORE_NUM_COLUMNS] =
         { "sound_channels",     NULL },
         { "input_service",      NULL },
         { "input_tilt",         NULL },
-        { "input_players",      NULL },
+        { "input_players",      columns_factory_input_players },
         { "input_buttons",      NULL },
         { "input_coins",        NULL },
-        { "driver_status",      NULL },
+        { "driver_status",      columns_factory_driver_status },
         { "driver_emulation",   NULL },
         { "driver_color",       NULL },
         { "driver_sound",       NULL },
@@ -292,10 +427,22 @@ static gchar *default_column_order[] =
         "description",
         "year",
         "manufacturer",
+        "driver_status",
+        "input_players",
         "name",
+        "sourcefile",
         "sampleset"
 };
 
+/**
+ * gva_columns_new_from_id:
+ * @column_id: the ID of the column to create
+ *
+ * Creates a new #GtkTreeViewColumn from the given @column_id, configured
+ * for use in the main tree view.
+ *
+ * Returns: a new #GtkTreeViewColumn
+ **/
 GtkTreeViewColumn *
 gva_columns_new_from_id (GvaGameStoreColumn column_id)
 {
@@ -314,6 +461,15 @@ gva_columns_new_from_id (GvaGameStoreColumn column_id)
         return column;
 }
 
+/**
+ * gva_columns_new_from_name:
+ * @column_name: the name of the column to create
+ *
+ * Creates a new #GtkTreeViewColumn from the given @column_name (as stored
+ * in GConf), configured for use in the main tree view.
+ *
+ * Returns: a new #GtkTreeViewColumn
+ **/
 GtkTreeViewColumn *
 gva_columns_new_from_name (const gchar *column_name)
 {
@@ -325,6 +481,16 @@ gva_columns_new_from_name (const gchar *column_name)
         return gva_columns_new_from_id (column_id);
 }
 
+/**
+ * gva_columns_lookup_id:
+ * @column_name: the name of the column to lookup
+ * @column_id: return location for the column ID
+ *
+ * Looks up the numeric column ID corresponding to @column_name, and write
+ * the result to @column_id if found.
+ *
+ * Returns: @TRUE if a column ID was found, @FALSE otherwise
+ **/
 gboolean
 gva_columns_lookup_id (const gchar *column_name,
                        GvaGameStoreColumn *column_id)
@@ -346,6 +512,14 @@ gva_columns_lookup_id (const gchar *column_name,
         return FALSE;
 }
 
+/**
+ * gva_columns_lookup_name:
+ * @column_id: the ID of the column to lookup
+ *
+ * Looks up the column name corresponding to @column_id.
+ *
+ * Returns: the column name, or @NULL if not found
+ **/
 const gchar *
 gva_columns_lookup_name (GvaGameStoreColumn column_id)
 {
@@ -372,6 +546,20 @@ columns_load_remove_name (GSList **p_list, const gchar *name)
         return TRUE;
 }
 
+/**
+ * gva_columns_load:
+ * @view: a #GtkTreeView
+ *
+ * Loads @view with columns in the order stored in the GConf key
+ * <filename>/apps/gnome-video-arcade/all-columns</filename>, but only
+ * makes visible those columns listed in
+ * <filename>/apps/gnome-video-arcade/columns</filename>.  Newly supported
+ * columns are appended to @view but remain invisible until explicitly
+ * selected in the Preferences window.
+ *
+ * Each column is loaded by reading the column name from GConf and passing
+ * it to gva_columns_new_from_name() to create the #GtkTreeViewColumn.
+ **/
 void
 gva_columns_load (GtkTreeView *view)
 {
@@ -399,6 +587,15 @@ gva_columns_load (GtkTreeView *view)
         gva_error_handle (&error);
         g_object_unref (client);
 
+        if (visible_columns == NULL)
+        {
+                /* Fall back to the default columns. */
+                visible_columns = g_slist_append (
+                        visible_columns, g_strdup ("favorite"));
+                visible_columns = g_slist_append (
+                        visible_columns, g_strdup ("description"));
+        }
+
         new_columns = NULL;
         for (ii = 0; ii < G_N_ELEMENTS (default_column_order); ii++)
         {
@@ -479,6 +676,15 @@ gva_columns_load (GtkTreeView *view)
         gva_columns_save (view);
 }
 
+/**
+ * gva_columns_save:
+ * @view: a #GtkTreeView
+ *
+ * Writes the column order and visible columns of @view to the GConf
+ * keys <filename>/apps/gnome-video-arcade/all-columns</filename> and
+ * <filename>/apps/gnome-video-arcade/columns</filename> respectively,
+ * using gva_columns_get_names() to extract the column names.
+ **/
 void
 gva_columns_save (GtkTreeView *view)
 {
@@ -512,6 +718,19 @@ gva_columns_save (GtkTreeView *view)
         g_object_unref (client);
 }
 
+/**
+ * gva_columns_get_names:
+ * @view: a #GtkTreeView
+ * @visible_only: only extract visible columns
+ *
+ * Extracts a list of column names from @view, using gva_columns_lookup_name()
+ * to convert each numeric column ID to a name.  If @visible_only is %TRUE
+ * then only visible columns are included in the list.  The column name
+ * strings are owned by @view and should not be freed; only the list itself
+ * should be freed using g_slist_free().
+ *
+ * Returns: a #GSList of column names
+ **/
 GSList *
 gva_columns_get_names (GtkTreeView *view,
                        gboolean visible_only)
@@ -540,3 +759,48 @@ gva_columns_get_names (GtkTreeView *view,
 
         return names;
 }
+
+/* Helper for gva_columns_get_names_full() */
+static void
+columns_add_dependency (GSList **p_list, const gchar *name)
+{
+        if (!g_slist_find_custom (*p_list, name, (GCompareFunc) strcmp))
+                *p_list = g_slist_prepend (*p_list, (gpointer) name);
+}
+
+/**
+ * gva_columns_get_names_full:
+ * @view: a #GtkTreeView
+ *
+ * Extracts a list of column names from @view, plus any additional column
+ * names from the game database necessary to render the tree view cells.
+ * The column name strings are owned by @view and should not be freed;
+ * only the list itself should be freed using g_slist_free().
+ *
+ * Returns: a #GSList of column names
+ **/
+GSList *
+gva_columns_get_names_full (GtkTreeView *view)
+{
+        GSList *names, *iter;
+
+        g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
+
+        names = gva_columns_get_names (view, FALSE);
+
+        /* XXX All the dependency information lives here for now.
+         *     It might make more sense in the column_info table,
+         *     with some private API for lookups. */
+
+        for (iter = names; iter != NULL; iter = iter->next)
+        {
+                /* XXX Nothing to do yet.
+                const gchar *column_name = iter->data;
+
+                if (strcmp (column_name, "description") == 0)
+                        columns_add_dependency (&names, "driver_status");
+                */
+        }
+
+        return names;
+}
