@@ -21,6 +21,7 @@
 #include <string.h>
 
 #include "gva-error.h"
+#include "gva-favorites.h"
 #include "gva-mame.h"
 #include "gva-util.h"
 
@@ -1063,7 +1064,6 @@ static ParserData *
 db_parser_data_new (GvaProcess *process)
 {
         ParserData *data;
-        gint errcode;
         GError *error = NULL;
 
         data = g_slice_new0 (ParserData);
@@ -1162,7 +1162,7 @@ db_parser_exit (GvaProcess *process,
                 gva_process_get_time_elapsed (process, &time_elapsed);
 
                 g_message (
-                        "Database built in %d.%d seconds.",
+                        "Database built in %ld.%ld seconds.",
                         time_elapsed.tv_sec, time_elapsed.tv_usec / 100000);
         }
         else
@@ -1174,79 +1174,6 @@ db_parser_exit (GvaProcess *process,
         db_parser_data_free (data);
 }
 
-static void
-db_verify_read (const gchar *name,
-                const gchar *status,
-                GHashTable *hash_table)
-{
-        GString *string;
-
-        /* Build a quoted, comma-separated list of games. */
-        string = g_hash_table_lookup (hash_table, status);
-        if (string != NULL)
-                g_string_append_printf (string, ", \"%s\"", name);
-        else
-        {
-                string = g_string_sized_new (1024);
-                g_string_printf (string, "\"%s\"", name);
-                g_hash_table_insert (hash_table, g_strdup (status), string);
-        }
-}
-
-/* Helper for db_verify_exit() */
-static void
-db_verify_foreach (const gchar *status,
-                   GString *names,
-                   const gchar *column)
-{
-        gchar *sql;
-        GError *error = NULL;
-
-        sql = g_strdup_printf (
-                "UPDATE game SET %s=\"%s\" WHERE name IN (%s)",
-                column, status, names->str);
-        gva_db_execute (sql, &error);
-        gva_error_handle (&error);
-        g_free (sql);
-}
-
-static void
-db_verify_exit (GvaProcess *process,
-                gint status,
-                GHashTable *results)
-{
-        const gchar *column;
-        GError *error = NULL;
-
-        if (process->error != NULL)
-                return;
-
-        column = g_object_get_data (G_OBJECT (process), "column");
-        g_assert (column != NULL);
-
-        gva_db_transaction_begin (&error);
-        gva_error_handle (&error);
-
-        g_hash_table_foreach (
-                results, (GHFunc) db_verify_foreach, (gpointer) column);
-
-        /* This part only really applies to romsets, but I suppose there's
-         * no harm in executing it twice. */
-        gva_db_execute (SQL_DELETE_NOT_FOUND, &error);
-        gva_error_handle (&error);
-
-        gva_db_transaction_commit (&error);
-        gva_error_handle (&error);
-
-        g_hash_table_destroy (results);
-}
-
-static void
-db_verify_string_free (GString *string)
-{
-        g_string_free (string, TRUE);
-}
-
 static gboolean
 db_create_tables (GError **error)
 {
@@ -1278,6 +1205,17 @@ db_function_isfavorite (sqlite3_context *context,
                 sqlite3_result_text (context, "no", -1, SQLITE_STATIC);
 }
 
+/**
+ * gva_db_init:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Opens the games database and creates the tables if they do not already
+ * exist.  If an error occurs, it returns %FALSE and sets @error.
+ *
+ * This function should be called once when the application starts.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_init (GError **error)
 {
@@ -1307,10 +1245,20 @@ fail:
         return FALSE;
 }
 
+/**
+ * gva_db_build:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Begins the lengthy process of populating the games database and returns a
+ * #GvaProcess to track it.  The database is populated by parsing detailed
+ * game information generated by MAME.  If an error occurs while starting the
+ * parsing process, it returns %NULL and sets @error.
+ *
+ * Returns: a new #GvaProcess, or %NULL
+ **/
 GvaProcess *
 gva_db_build (GError **error)
 {
-        const gchar *filename;
         GvaProcess *process;
         ParserData *data;
 
@@ -1409,64 +1357,16 @@ gva_db_build (GError **error)
         return process;
 }
 
-GvaProcess *
-gva_db_verify_romsets (GError **error)
-{
-        GvaProcess *process;
-        GHashTable *results;
-
-        results = g_hash_table_new_full (
-                g_str_hash, g_str_equal,
-                (GDestroyNotify) g_free,
-                (GDestroyNotify) db_verify_string_free);
-
-        process = gva_mame_verify_roms (
-                (GvaMameCallback) db_verify_read, results, error);
-
-        if (process == NULL)
-        {
-                g_hash_table_destroy (results);
-                return NULL;
-        }
-
-        g_object_set_data (G_OBJECT (process), "column", "romset");
-
-        g_signal_connect (
-                process, "exited",
-                G_CALLBACK (db_verify_exit), results);
-
-        return process;
-}
-
-GvaProcess *
-gva_db_verify_samplesets (GError **error)
-{
-        GvaProcess *process;
-        GHashTable *results;
-
-        results = g_hash_table_new_full (
-                g_str_hash, g_str_equal,
-                (GDestroyNotify) g_free,
-                (GDestroyNotify) db_verify_string_free);
-
-        process = gva_mame_verify_samples (
-                (GvaMameCallback) db_verify_read, results, error);
-
-        if (process == NULL)
-        {
-                g_hash_table_destroy (results);
-                return NULL;
-        }
-
-        g_object_set_data (G_OBJECT (process), "column", "sampleset");
-
-        g_signal_connect (
-                process, "exited",
-                G_CALLBACK (db_verify_exit), results);
-
-        return process;
-}
-
+/**
+ * gva_db_reset:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Removes all game information from the database and restores the tables
+ * to their initial state.  If an error occurs, it returns %FALSE and sets
+ * @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_reset (GError **error)
 {
@@ -1485,6 +1385,18 @@ gva_db_reset (GError **error)
         return gva_db_init (error);
 }
 
+/**
+ * gva_db_execute:
+ * @sql: an SQL statement
+ * @error: return location for a #GError, or %NULL
+ *
+ * Executes the given SQL statement without returning any results.  This
+ * function is appropriate for one-off operations like "CREATE TABLE" or
+ * "BEGIN TRANSACTION".  If an error occurs, it returns %FALSE and sets
+ * @error.
+ *
+ * Returns: %TRUE on succes, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_execute (const gchar *sql,
                 GError **error)
@@ -1506,24 +1418,62 @@ gva_db_execute (const gchar *sql,
         return (errcode == SQLITE_OK);
 }
 
+/**
+ * gva_db_transaction_begin:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Convenience function begins a database transaction.  If an error occurs,
+ * it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_transaction_begin (GError **error)
 {
         return gva_db_execute ("BEGIN TRANSACTION", error);
 }
 
+/**
+ * gva_db_transaction_commit:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Convenience function commits a database transaction.  If an error occurs,
+ * it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_transaction_commit (GError **error)
 {
         return gva_db_execute ("COMMIT TRANSACTION", error);
 }
 
+/**
+ * gva_db_transaction_rollback:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Convenience function rolls back a database transaction.  If an error
+ * occurs, it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_transaction_rollback (GError **error)
 {
         return gva_db_execute ("ROLLBACK TRANSACTION", error);
 }
 
+/**
+ * gva_db_prepare:
+ * @sql: an SQL statement
+ * @stmt: return location for a compiled statement handle
+ * @error: return location for a #GError, or %NULL
+ *
+ * Compiles the given SQL statement and assigns the resulting statement
+ * handle to *@stmt.  If an error occurs, it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_prepare (const gchar *sql,
                 sqlite3_stmt **stmt,
@@ -1560,6 +1510,17 @@ db_get_build_cb (gpointer user_data,
         return 0;
 }
 
+/**
+ * gva_db_get_build:
+ * @build: return location for the build ID
+ * @error: return location for a #GError, or %NULL
+ *
+ * Returns the build ID of the MAME executable from which the contents of the
+ * database was generated.  If an error occurs, it returns %FALSE and sets
+ * @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_get_build (gchar **build,
                   GError **error)
@@ -1583,6 +1544,13 @@ gva_db_get_build (gchar **build,
         return (errcode == SQLITE_OK);
 }
 
+/**
+ * gva_db_get_filename:
+ *
+ * Returns the abolute path of the games database.
+ *
+ * Returns: filename for the games database
+ **/
 const gchar *
 gva_db_get_filename (void)
 {
@@ -1595,10 +1563,21 @@ gva_db_get_filename (void)
         return filename;
 }
 
+/**
+ * gva_db_needs_rebuilt:
+ * 
+ * Runs a series of tests to determine whether the game database is
+ * out-of-date and needs to be rebuilt.  Examples of conditions that
+ * would cause a database rebuild are a new version of
+ * <emphasis>GNOME Video Arcade</emphasis>, a new version of MAME,
+ * database corruption, or if the user explicitly asked us to rebuild.
+ *
+ * Returns: %TRUE if a rebuild is needed, %FALSE if the database
+ *          <emphasis>seems</emphasis> okay
+ **/
 gboolean
 gva_db_needs_rebuilt (void)
 {
-        GConfClient *client;
         gchar *db_build_id = NULL;
         gchar *mame_version = NULL;
         const gchar *reason;
@@ -1647,6 +1626,18 @@ exit:
         return rebuild;
 }
 
+/**
+ * gva_db_set_error:
+ * @error: return location for a #GError, or %NULL
+ * @code: SQLite error code
+ * @message: SQLite error message, or %NULL
+ *
+ * Does nothing if @error is %NULL; if @error is non-%NULL, then *@error must
+ * be %NULL.  Converts an SQLite error code and error message to a #GError
+ * and assigns it to *@error with a domain of %GVA_SQLITE_ERROR.  If @message
+ * is %NULL, the function calls sqlite3_errcode() to obtain the error code and
+ * sqlite3_errmsg() to obtain the error message.
+ **/
 void
 gva_db_set_error (GError **error,
                   gint code,
