@@ -23,13 +23,21 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef WITH_GNOME
+#include <gnome.h>
+#endif
+
+#include "gva-audit.h"
 #include "gva-db.h"
+#include "gva-error.h"
 #include "gva-history.h"
 #include "gva-main.h"
 #include "gva-mame.h"
 #include "gva-play-back.h"
 #include "gva-preferences.h"
 #include "gva-properties.h"
+#include "gva-search.h"
+#include "gva-tree-view.h"
 #include "gva-ui.h"
 
 static GOptionEntry entries[] =
@@ -38,6 +46,15 @@ static GOptionEntry entries[] =
           G_OPTION_ARG_NONE, &opt_build_database,
           N_("Build the games database"), NULL },
 
+        { "inspect", 'i', 0,
+          G_OPTION_ARG_STRING, &opt_inspect,
+          N_("Inspect an emulator setting"),
+          N_("NAME") },
+
+        { "version", 'v', 0,
+          G_OPTION_ARG_NONE, &opt_version,
+          N_("Show the application version"), NULL },
+
         { "which-emulator", 'w', 0,
           G_OPTION_ARG_NONE, &opt_which_emulator,
           N_("Show which emulator will be used"), NULL },
@@ -48,10 +65,8 @@ static GOptionEntry entries[] =
 static void
 start (void)
 {
-        GvaProcess *process = NULL;
         gchar *mame_version;
         guint context_id;
-        gint last_view;
         GError *error = NULL;
 
         context_id = gva_main_statusbar_get_context_id (G_STRFUNC);
@@ -66,19 +81,12 @@ start (void)
 
         if (gva_db_needs_rebuilt ())
         {
-                process = gva_main_build_database (&error);
+                gboolean success;
+
+                success = gva_main_build_database (&error);
                 gva_error_handle (&error);
-        }
 
-        if (process != NULL)
-        {
-                gboolean main_loop_quit;
-
-                while (!gva_process_has_exited (process, NULL))
-                        main_loop_quit = gtk_main_iteration ();
-                g_object_unref (process);
-
-                if (main_loop_quit)
+                if (!success)
                         return;
         }
 
@@ -86,32 +94,73 @@ start (void)
         gtk_action_set_sensitive (GVA_ACTION_VIEW_FAVORITES, TRUE);
         gtk_action_set_sensitive (GVA_ACTION_VIEW_RESULTS, TRUE);
 
+        gconf_bridge_bind_property (
+                gconf_bridge_get (), GVA_GCONF_SELECTED_VIEW_KEY,
+                G_OBJECT (GVA_ACTION_VIEW_AVAILABLE), "current-value");
+
         /* Force a tree view update. */
-        last_view = gva_tree_view_get_last_selected_view ();
-        if (last_view == gva_tree_view_get_selected_view ())
+        if (gva_tree_view_get_selected_view () == 0)
         {
                 gva_tree_view_update (&error);
                 gva_error_handle (&error);
         }
-        else
-                gva_tree_view_set_selected_view (last_view);
 }
 
 gint
 main (gint argc, gchar **argv)
 {
+#ifdef WITH_GNOME
+        GnomeProgram *program;
+        GOptionContext *context;
+#endif
         GError *error = NULL;
 
+        g_thread_init (NULL);
+
         bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
-        g_thread_init (NULL);
+#ifdef WITH_GNOME
+        context = g_option_context_new (NULL);
+        g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
+        g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 
+        program = gnome_program_init (
+                PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
+                GNOME_PROGRAM_STANDARD_PROPERTIES,
+                GNOME_PARAM_GOPTION_CONTEXT, context,
+                GNOME_PARAM_NONE);
+#else
         gtk_init_with_args (
                 &argc, &argv, NULL, entries, GETTEXT_PACKAGE, &error);
         if (error != NULL)
                 g_error ("%s", error->message);
+#endif
+
+        if (opt_inspect != NULL)
+        {
+                gchar *value;
+
+                value = gva_mame_get_config_value (opt_inspect, &error);
+                if (value != NULL)
+                {
+                        g_print ("%s\n", value);
+                        g_free (value);
+                }
+                else
+                {
+                        g_printerr ("%s\n", error->message);
+                        g_clear_error (&error);
+                }
+                exit (0);
+        }
+
+        if (opt_version)
+        {
+                g_print ("%s\n", PACKAGE_STRING);
+                exit (0);
+        }
 
         if (opt_which_emulator)
         {
@@ -124,10 +173,12 @@ main (gint argc, gchar **argv)
         if (!gva_db_init (&error))
                 g_error ("%s", error->message);
 
+        gva_audit_init ();
         gva_main_init ();
         gva_play_back_init ();
         gva_preferences_init ();
         gva_properties_init ();
+        gva_search_init ();
 
 #ifdef HISTORY_FILE
         gva_history_init (&error);
@@ -138,5 +189,9 @@ main (gint argc, gchar **argv)
 
         gtk_main ();
 
+#ifdef WITH_GNOME
+        g_object_unref (program);
+#endif
+
         return EXIT_SUCCESS;
 }
